/* Copyright 2013 Sven Jacobs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.svenjacobs.lugaene;

import org.apache.lucene.store.IndexInput;

import java.io.IOException;

import static com.svenjacobs.lugaene.GaeFileBufferEntity.BUFFER_SIZE;

/**
 * Input from a {@link GaeFileEntity}
 *
 * @author Sven Jacobs
 */
public class GaeIndexInput extends IndexInput {

    private GaeIndexMixin mixin;

    GaeIndexInput(final GaeFileEntity file) {
        super("GaeIndexInput(name=" + file.name + ")");

        this.mixin = new GaeIndexMixin(file);
    }

    @Override
    public void close() throws IOException {
    }

    @Override
    public long getFilePointer() {
        return mixin.getFilePointer();
    }

    @Override
    public void seek(final long pos) throws IOException {
        mixin.currentBufferIndex = (int) (pos / BUFFER_SIZE);
        mixin.bufferPos = (int) (pos % BUFFER_SIZE);
        mixin.loadBuffer();
    }

    @Override
    public long length() {
        return mixin.length();
    }

    @Override
    public byte readByte() throws IOException {
        if (mixin.bufferPos == BUFFER_SIZE) {
            mixin.nextBuffer();
        }

        return mixin.currentBuffer()[mixin.bufferPos++];
    }

    @Override
    public void readBytes(final byte[] b,
                          int offset,
                          int length) throws IOException {

        while (length > 0) {
            int remainingBytesInBuffer = BUFFER_SIZE - mixin.bufferPos;

            if (remainingBytesInBuffer == 0) {
                mixin.nextBuffer();
                remainingBytesInBuffer = BUFFER_SIZE;
            }

            final int bytesToCopy = Math.min(length, remainingBytesInBuffer);

            System.arraycopy(mixin.currentBuffer(), mixin.bufferPos, b, offset, bytesToCopy);

            offset += bytesToCopy;
            length -= bytesToCopy;
            mixin.bufferPos += bytesToCopy;
        }
    }

    /**
     * Very evil but we need to support clone because Lucene uses it.
     *
     * @return Clone of GaeIndexInput
     */
    @Override
    public IndexInput clone() {
        super.clone(); // Just to obey contract of clone

        final GaeIndexInput clone = new GaeIndexInput(mixin.file);

        try {
            clone.seek(mixin.getFilePointer());
        } catch (IOException ignored) {
        }

        return clone;
    }
}
